#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 29 15:11:52 2022

@author: lioneltruflandier
"""

from numpy import amax, amin, matmul, array, trapz, size, exp
from ase.units import Hartree, Rydberg

# compute occupation numbers at G and Z assuming Fermi-Dirac distribution

###############################################################################
#
# Given eigenvalues and Fermi level output occupation numbers
#
###############################################################################
def print_occ(eigenvalues,fermi,kT,method,label):

    occ_num = [ ]
    
    if (method == 'aufbau'):
       for eig in eigenvalues:
            if ( eig <= fermi):
                occ_num.append( 1.0 )
            else:
                occ_num.append( 0.0 )

    elif (method == 'fermi-dirac'):
        for eig in eigenvalues:
            tmp = (eig - fermi)/Rydberg/kT
            if   ( tmp >  400 ):
                occ_num.append( 0.0 )
            elif ( tmp < -400 ):
                occ_num.append( 1.0 )
            else:
                occ_num.append( 1.0/( exp(tmp) + 1.0 ) )
    else:        
        print('{}'.format('Error: method unrocognized... '))
    
    print()
    print('{}'.format('Band energies at the {} point'.format(label)))
    print('{}'.format('# energy in eV (occupation number)'))
    
    occ_tot = 0.0
    for eig, occ in zip(reversed(eigenvalues),reversed(occ_num)):
        print('{:14.5f} ({:5.3f})'.format(eig,occ*2))
        occ_tot = occ_tot + occ*2
    print('{} {:5.3f})'.format('(total'.rjust(21),occ_tot)) 


###############################################################################
#
# Given QEspresso output (ASE Atoms object) print data
#
###############################################################################
def qe_print_output_data(struct,verbose):
    # results are extracted from struct.calc.output !
    dft_energy   = struct.get_potential_energy()
    #fermi_energy = struct.calc.output.get_fermi_level()
    atom_names   = struct.get_chemical_symbols()
    atom_forces  = struct.get_forces()
    stress_tens  = struct.get_stress()
    stress_tens_comp = ['xx', 'yy', 'zz', 'yz', 'xz', 'xy']
    
    print('Total electronic energy = {:14.6f} eV'.format(dft_energy))
    print('Atoms kinetic energy    = {:14.6f} eV'.format(struct.get_kinetic_energy())) # of the atoms (relevant only with MD)
    print('Total energy = {:14.6f} eV'.format(struct.get_total_energy())) # total electronic + atoms-kinetic energy
    #print('Fermi energy = {:14.6f} eV'.format(fermi_energy)) # Fermi level
    print()
    if ( verbose > 0):
        print('Atomic forces (eV/Ang)')
        for i in range(len(struct)):
            print('  {}{:12.6f}{:12.6f}{:12.6f}'.format(atom_names[i],atom_forces[i,0],atom_forces[i,1],atom_forces[i,2]))
        print()
        
    print('Cell stress tensor components (eV/Ang**3)')
    for i in range(5):
        print('  {} = {:11.6f}'.format(stress_tens_comp[i],stress_tens[i]))


###############################################################################
#
# Given ASE Atoms object print data
#
###############################################################################
def print_struct_data(struct,verbose):
    
    atom_names = struct.get_chemical_symbols()
    atom_positions_cart = struct.get_positions(wrap=False)
    atom_positions_cell = struct.get_scaled_positions(wrap=False)
    
    cell_param = struct.get_cell_lengths_and_angles()
    cell_tens  = struct.get_cell()

    if ( verbose > 0):

        print('Cartesian atomic positions (Ang.)')
        for i in range(len(struct)):
            # print formated cartesian
            print('  {:2}{:14.6f}{:14.6f}{:14.6f}'.format(atom_names[i],
                atom_positions_cart[i,0],atom_positions_cart[i,1],atom_positions_cart[i,2]))
        
        print()
        print('Fractional atomic positions (Adim.)')
        for i in range(len(struct)):
            # print formated fractional
            print('  {:2}{:14.6f}{:14.6f}{:14.6f}'.format(atom_names[i],
                atom_positions_cell[i,0],atom_positions_cell[i,1],atom_positions_cell[i,2]))
        
        # Check the Cartesian positions are recovered from the cell tensor
        print()
        print('Cell tensor (Ang.)')
        for i in range(len(cell_tens)):
            print('    {:14.6f}{:14.6f}{:14.6f}'.format(cell_tens[i,0],\
                                                        cell_tens[i,1],\
                                                        cell_tens[i,2]))
        print()
        print('Cartesian atomic positions (Ang.) from cell tensor')
        for i in range(len(struct)):
            # WARNING: xyz -> row
            tmp_positions = matmul(atom_positions_cell[i,:],cell_tens)
            print('  {:2}{:14.6f}{:14.6f}{:14.6f}'.format(atom_names[i],\
                    tmp_positions[0],tmp_positions[1],tmp_positions[2]))
    
    print()
    print('Cell parameters (Ang. and degree)')
    print('  a = {:14.6f}'.format(cell_param[0]))
    print('  b = {:14.6f}'.format(cell_param[1]))
    print('  c = {:14.6f}'.format(cell_param[2]))
    print('  alpha = {:8.4f}'.format(cell_param[3]))
    print('  beta  = {:8.4f}'.format(cell_param[4]))
    print('  gamma = {:8.4f}'.format(cell_param[5]))